'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/**
 * Renders simple values (eg. strings, numbers, booleans, etc)
 */

var replaces = {
  null: null,
  true: true,
  false: false,
  undefined: undefined
};

var transformString = function transformString(value) {
  var replaced = Object.prototype.hasOwnProperty.call(replaces, value) ? replaces[value] : value;
  return !isNaN(parseFloat(replaced)) && isFinite(replaced) ? Number(replaced) : replaced;
};

var getString = function getString(_ref) {
  var value = _ref.value,
      nodeType = _ref.nodeType,
      valueGetter = _ref.valueGetter;

  if (nodeType === 'String') {
    return value;
  }
  return valueGetter(value);
};

var JSONValueNode = function (_React$Component) {
  _inherits(JSONValueNode, _React$Component);

  _createClass(JSONValueNode, null, [{
    key: 'getDerivedStateFromProps',
    value: function getDerivedStateFromProps(props, state) {
      // If the props value changes and we're not currently editing,
      // it should be reflected in this component.
      if (props.value !== state.lastValue && !state.editing) {
        var value = getString(props);
        return {
          value: value,
          lastValue: value
        };
      }
      return null;
    }
  }]);

  function JSONValueNode(props) {
    _classCallCheck(this, JSONValueNode);

    var _this = _possibleConstructorReturn(this, (JSONValueNode.__proto__ || Object.getPrototypeOf(JSONValueNode)).call(this, props));

    _this.keydown = function (e) {
      // Echap, remove edit mode and reset value
      if (e.keyCode === 27) {
        _this.setState({ editing: false, value: getString(_this.props) });
      }

      // If it's not enter, do nothing
      if (e.keyCode !== 13) {
        return;
      }

      var _this$props = _this.props,
          onChange = _this$props.onChange,
          keyPath = _this$props.keyPath;
      var value = _this.state.value;


      onChange({ keyPath: keyPath, value: transformString(value) });
      _this.setState({ editing: false });
    };

    _this.updateValue = function (e) {
      return _this.setState({ value: e.target.value });
    };

    _this.toggleEdit = function () {
      return _this.setState({ editing: !_this.state.editing });
    };

    _this.state = {
      value: getString(props),
      lastValue: getString(props), // For gDSFP
      editing: false
    };
    return _this;
  }

  _createClass(JSONValueNode, [{
    key: 'render',
    value: function render() {
      var _props = this.props,
          nodeType = _props.nodeType,
          styling = _props.styling,
          labelRenderer = _props.labelRenderer,
          keyPath = _props.keyPath,
          valueRenderer = _props.valueRenderer,
          valueGetter = _props.valueGetter;
      var _state = this.state,
          value = _state.value,
          editing = _state.editing;

      var isEditable = ['Null', 'Undefined', 'String', 'Number', 'Boolean'].includes(nodeType);

      return _react2.default.createElement(
        'li',
        styling('value', nodeType, keyPath),
        _react2.default.createElement(
          'label',
          styling(['label', 'valueLabel'], nodeType, keyPath),
          labelRenderer(keyPath, nodeType, false, false)
        ),
        editing ? _react2.default.createElement('input', {
          onChange: this.updateValue,
          onKeyDown: this.keydown,
          style: _extends({}, styling('valueText', nodeType, keyPath).style, {
            width: '100%',
            overflow: 'hidden'
          }),
          value: value,
          type: 'text',
          autoFocus: true
        }) : _react2.default.createElement(
          'span',
          _extends({
            onClick: isEditable && this.toggleEdit
          }, styling('valueText', nodeType, keyPath)),
          valueRenderer.apply(undefined, [valueGetter(this.props.value), value].concat(_toConsumableArray(keyPath)))
        )
      );
    }
  }]);

  return JSONValueNode;
}(_react2.default.Component);

JSONValueNode.propTypes = {
  nodeType: _propTypes2.default.string.isRequired,
  styling: _propTypes2.default.func.isRequired,
  labelRenderer: _propTypes2.default.func.isRequired,
  keyPath: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])).isRequired,
  valueRenderer: _propTypes2.default.func.isRequired,
  value: _propTypes2.default.any,
  valueGetter: _propTypes2.default.func,
  onChange: _propTypes2.default.func.isRequired
};
JSONValueNode.defaultProps = {
  valueGetter: function valueGetter(value) {
    return value;
  }
};
exports.default = JSONValueNode;